@@ -18,6 +18,13 @@ logger = get_logger("RPC_CLIENT")
 def isNotInvalidStatusCode(value):
     return not isinstance(value, InvalidStatusCode)
 
+def isNotForbbiden(value)->bool:
+    """
+    Returns:
+        bool: Returns True as long as the given exception value is not InvalidStatusCode with 401 or 403
+    """
+    return not (isinstance(value, InvalidStatusCode) and (value.status_code == 401 or value.status_code == 403))
+
 
 class WebSocketRpcClient:
     """
@@ -30,8 +37,8 @@ class WebSocketRpcClient:
         logger.exception(retry_state.outcome.exception())
 
     DEFAULT_RETRY_CONFIG = {
-        'wait': wait.wait_random_exponential(),
-        'retry': retry_if_exception(isNotInvalidStatusCode),
+        'wait': wait.wait_random_exponential(min=0.1, max=120),
+        'retry': retry_if_exception(isNotForbbiden),
         'reraise': True,
         "retry_error_callback":logerror
     }
@@ -128,6 +135,9 @@ class WebSocketRpcClient:
         except ConnectionClosedOK:
             logger.info("RPC connection closed")
             raise
+        except InvalidStatusCode as err:
+            logger.info(f"RPC Websocket failed - with invalid status code {err.status_code}")
+            raise
         except WebSocketException as err:
             logger.info(f"RPC Websocket failed - with {err}")
             raise
@@ -232,12 +242,15 @@ class WebSocketRpcClient:
         """
         Join on the internal reader task
         """
-        await self._read_task
+        try:
+            await self._read_task
+        except asyncio.CancelledError:
+            logger.info(f"RPC Reader task was cancelled.")
 
     async def call(self, name, args={}, timeout=None):
         """
         Call a method and wait for a response to be received
-         Args: 
+         Args:
             name (str): name of the method to call on the other side (As defined on the otherside's RpcMethods object)
             args (dict): keyword arguments to be passeds to otherside method
         """
